/**
 * Copyright (C) 2012 Yvon TANGUY
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.download;

import java.net.URL;
import java.util.Date;

import org.vono.narau.Common;
import org.vono.narau.R;
import org.vono.narau.dictionary.DictionaryDB;
import org.vono.narau.kanji.KanjiDB;

/**
 * filename size file_time sha512 db_version download_url
 * 
 */
public class DBInfos {

	/**
	 * Extension of compressed databases file to download.
	 */
	private static final String GZ_EXT = ".gz";

	/**
	 * gzip filename.
	 */
	public final String name;
	/**
	 * sqlite db filename.
	 */
	public final String dbName;
	public final int progressBarId;
	public final int checkBoxId;
	public final int layoutId;
	public final int textViewDateId;
	public final int strDwnlNowId;
	public final int expectedDBVersion;

	public long gzSize;
	public long dbSize;
	public Date creationDate;
	public String sha512;
	public int dbVersion;
	public URL[] url;
	public boolean infosLoaded;
	public boolean selected;

	public static final DBInfos[] arrayDBInfos = new DBInfos[2];

	static {
		DBInfos infos = new DBInfos(Common.DB_DICT_NAME, R.id.downloadProgressBarDict,
				R.id.dwnlNowCheckBoxDictDB, R.id.downloadLayoutDictionary,
				R.id.dwnlNowTextViewDictDBDate, R.string.dwnlNowDictDB,
				DictionaryDB.VALUE_DB_VERSION);
		arrayDBInfos[0] = infos;
		infos = new DBInfos(Common.DB_KANJI_NAME, R.id.downloadProgressBarKanji,
				R.id.dwnlNowCheckBoxKanjiDB, R.id.downloadLayoutKanji,
				R.id.dwnlNowTextViewKanjiDBDate, R.string.dwnlNowKanjiDB, KanjiDB.VALUE_DB_VERSION);
		arrayDBInfos[1] = infos;
	}

	private DBInfos(String name, int progressBarId, int checkBoxId, int layoutId,
			int textViewDateId, int strDwnlNowId, int expectedDBVersion) {
		this.dbName = name;
		this.name = name + GZ_EXT;
		this.progressBarId = progressBarId;
		this.checkBoxId = checkBoxId;
		this.layoutId = layoutId;
		this.textViewDateId = textViewDateId;
		this.strDwnlNowId = strDwnlNowId;
		this.expectedDBVersion = expectedDBVersion;
		this.infosLoaded = false;
		this.dbSize = 0;
		this.gzSize = 0;
		this.creationDate = null;
		this.selected = true;
	}

}
