/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.cupcake.download;

import java.text.SimpleDateFormat;
import java.util.ArrayList;

import org.vono.narau.cupcake.Common;
import org.vono.narau.cupcake.MenuActivity;
import org.vono.narau.cupcake.R;

import android.app.Activity;
import android.app.ProgressDialog;
import android.os.Bundle;
import android.os.Environment;
import android.os.StatFs;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

/**
 * Manage the download database process (first run & update).
 * 
 */
public class DownloaderActivity extends Activity implements OnClickListener {
	public static final String UPDATE_PROCESS = "upd";

	private enum CurrentScreen {
		NONE, FIRST_RUN, QUESTION, DOWNLOAD, DOWNLOAD_DONE, QUIT;
	}

	/**
	 * The current download screen.
	 */
	private static CurrentScreen currentScreen = CurrentScreen.NONE;
	private static boolean updateDatabase = false;

	private ProgressDialog progressDialog;
	private TextView textViewDownloadSpeed;

	/** Called when the activity is created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		switch (DownloaderActivity.currentScreen) {
		case NONE:
			DownloaderActivity.updateDatabase = super.getIntent().getBooleanExtra(UPDATE_PROCESS,
					false);

			if (Environment.MEDIA_MOUNTED_READ_ONLY.equals(Environment.getExternalStorageState())) {
				// Error we can't write a file in a read only mounted
				// storage.
				super.setContentView(R.layout.menu_init_error);
				TextView tv = (TextView) super.findViewById(R.id.initErrorTextView);
				tv.setText(R.string.initSdcardReadOnly);
				Button button = (Button) super.findViewById(R.id.initErrorButtonQuit);
				button.setOnClickListener(this);
			} else if (DownloaderActivity.updateDatabase) {
				this.firstRunContinue(null);
			} else {
				this.showFirstRunScreen();
			}
			break;
		case FIRST_RUN:
			this.showFirstRunScreen();
			break;
		case QUESTION:
			this.showDownloadViewQuestion();
			break;
		case DOWNLOAD:
			this.showDownloadView();
			break;
		case DOWNLOAD_DONE:
			this.showDownloadView();
			this.downloadComplete();
			break;
		case QUIT:
			super.finish();
		}
	}

	protected void onDestroy() {
		super.onDestroy();
		DownloaderGzipDB.updateUIActivity(null);
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {

		if (KeyEvent.KEYCODE_BACK == keyCode) {
			switch (DownloaderActivity.currentScreen) {
			case FIRST_RUN:
			case QUESTION:
				if (DownloaderActivity.updateDatabase) {
					super.setResult(Activity.RESULT_OK);
				} else {
					super.setResult(Activity.RESULT_CANCELED);
				}
				break;
			case DOWNLOAD:
				super.setResult(MenuActivity.RESULT_PENDING);
				break;
			case DOWNLOAD_DONE:
				super.setResult(Activity.RESULT_OK);
				break;
			default:
				break;
			}
		}

		return super.onKeyDown(keyCode, event);
	}

	private void showFirstRunScreen() {
		super.setContentView(R.layout.download_firstrun);
		DownloaderActivity.currentScreen = CurrentScreen.FIRST_RUN;
		Button button = (Button) super.findViewById(R.id.downloadFirstRunButtonContinue);
		button.setOnClickListener(this);
		button = (Button) super.findViewById(R.id.downloadFirstRunButtonQuit);
		button.setOnClickListener(this);
	}

	private void showDownloadViewQuestion() {
		super.setContentView(R.layout.download_question);
		DownloaderActivity.currentScreen = CurrentScreen.QUESTION;

		final int numDownloads = DBInfos.arrayDBInfos.length;
		for (int i = 0; i < numDownloads; i++) {
			final DBInfos dbInfo = DBInfos.arrayDBInfos[i];
			CheckBox cb = (CheckBox) super.findViewById(dbInfo.checkBoxId);

			String cbText = String.format(super.getString(dbInfo.strDwnlNowId), this.bytesToHumanSize(dbInfo.gzSize),
					this.bytesToHumanSize(dbInfo.dbSize));
			cb.setText(cbText);
			cb.setChecked(dbInfo.selected);
			cb.setOnClickListener(this);
			Button button = (Button) super.findViewById(R.id.dwnlNowButtonOk);
			button.setOnClickListener(this);
			button = (Button) super.findViewById(R.id.dwnlNowButtonCancel);
			button.setOnClickListener(this);

			TextView tv = (TextView) super.findViewById(dbInfo.textViewDateId);
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
			tv.setText(super.getString(R.string.downloadDBUpdateDate, dbInfo.dbName, sdf.format(dbInfo.creationDate)));
		}

		this.clickCheckBoxDownload(null);
	}

	/**
	 * Show the download view
	 */
	private void showDownloadView() {
		super.setContentView(R.layout.download);
		DownloaderActivity.currentScreen = CurrentScreen.DOWNLOAD;
		this.textViewDownloadSpeed = (TextView) super.findViewById(R.id.downloadTextViewSpeed);
		DownloaderGzipDB.updateUIActivity(this);

		Button button = (Button) super.findViewById(R.id.downloadButtonCancel);
		button.setOnClickListener(this);
		button = (Button) super.findViewById(R.id.downloadButtonContinue);
		button.setOnClickListener(this);
	}

	/**
	 * menu_download_question.xml<br/>
	 * Click on ok or cancel button in the download now view.
	 * 
	 * @param v
	 *            the clicked button
	 */
	public void dwnlNowClickButton(View v) {

		this.showDownloadView();

		final int numDwnl = DBInfos.arrayDBInfos.length;
		ArrayList<Integer> listIds = new ArrayList<Integer>(numDwnl);

		for (int i = 0; i < numDwnl; i++) {
			final DBInfos dbInfos = DBInfos.arrayDBInfos[i];
			LinearLayout ll = (LinearLayout) super.findViewById(dbInfos.layoutId);
			if (dbInfos.selected) {
				listIds.add(i);
				ll.setVisibility(View.VISIBLE);
			} else {
				// Don't display the progress bar of what we don't download.
				ll.setVisibility(View.GONE);
			}
		}

		DownloaderGzipDB.download(this, listIds);
	}

	/**
	 * Click on the continue button in the first run layout.
	 * 
	 * @param v
	 *            the button
	 */
	public void firstRunContinue(View v) {
		super.setContentView(R.layout.menu_init_error);
		final String waitMessage = super.getString(R.string.downloadDBInfo);
		this.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING, waitMessage, true);
		DownloaderDBInfos.startDownload(this);
	}

	/**
	 * 
	 * @param v
	 *            The continue button on the download database view.
	 */
	public void showMainMenu(View v) {
		DownloaderActivity.currentScreen = CurrentScreen.DOWNLOAD_DONE;
		super.setResult(Activity.RESULT_OK);
		super.finish();
	}

	public void dbInfoJobDone(boolean retValue, String message) {
		this.progressDialog.dismiss();
		this.progressDialog = null;
		if (retValue) {
			this.showDownloadViewQuestion();
		} else {
			TextView tv = (TextView) super.findViewById(R.id.initErrorTextView);
			tv.setText(message);
			Button button = (Button) super.findViewById(R.id.initErrorButtonQuit);
			button.setOnClickListener(this);
		}
	}

	/**
	 * menu_download_question.xml<br/>
	 * Update the total download size, when click on a checkbox.
	 * 
	 * @param v
	 *            The checkbox clicked
	 */
	public void clickCheckBoxDownload(View v) {
		StatFs stat = new StatFs(Environment.getExternalStorageDirectory().getPath());
		long sdAvailSize = stat.getAvailableBlocks() * stat.getBlockSize();

		long dwnlSize = 0;
		long needSize = 0;

		final int numDownloads = DBInfos.arrayDBInfos.length;
		for (int i = 0; i < numDownloads; i++) {
			final DBInfos dbInfo = DBInfos.arrayDBInfos[i];
			CheckBox cb = (CheckBox) super.findViewById(dbInfo.checkBoxId);
			if (cb.isChecked()) {
				dwnlSize += dbInfo.gzSize;
				needSize += dbInfo.dbSize;
				dbInfo.selected = true;
			} else {
				dbInfo.selected = false;
			}
		}

		String text = String.format(super.getString(R.string.dwnlNowTotal), this.bytesToHumanSize(dwnlSize));
		TextView tv = (TextView) super.findViewById(R.id.dwnlNowTextViewTotal);
		tv.setText(text);

		tv = (TextView) super.findViewById(R.id.dwnlNowTextViewWarnLowSpace);
		if (sdAvailSize <= needSize) {
			text = String.format(super.getString(R.string.dwnlNowWarnLowSpace), this.bytesToHumanSize(sdAvailSize),
					this.bytesToHumanSize(needSize));
			tv.setText(text);
			tv.setVisibility(View.VISIBLE);
		} else {
			tv.setVisibility(View.GONE);
		}

		Button bt = (Button) super.findViewById(R.id.dwnlNowButtonOk);
		if (0 == needSize || sdAvailSize <= needSize) {
			// If no item selected, disable the ok button
			// or if there is not enough space on the sdcard
			bt.setEnabled(false);
		} else {
			bt.setEnabled(true);
		}
	}

	/**
	 * Handle a press on a quit/cancel button:<br/>
	 * <ul>
	 * <li>on the first run screen</li>
	 * <li>on the question screen</li>
	 * <li>on the download screen.</li>
	 * </ul>
	 * 
	 * @param v
	 *            a button
	 */
	public void quitApplication(View v) {
		if (DownloaderActivity.updateDatabase) {
			switch (v.getId()) {
			case R.id.dwnlNowButtonCancel:
				super.setResult(Activity.RESULT_OK);
				DownloaderActivity.currentScreen = CurrentScreen.NONE;
				break;
			}
		} else {
			if (null != this.progressDialog) {
				this.progressDialog.dismiss();
				this.progressDialog = null;
			}

			if (DownloaderGzipDB.isDownloading()) {
				DownloaderGzipDB.prematureThreadStop();
			}

			super.setResult(Activity.RESULT_CANCELED);
			DownloaderActivity.currentScreen = CurrentScreen.QUIT;
		}

		super.finish();
	}

	/**
	 * Update the ui with the download informations.
	 * 
	 * @param id
	 *            if of the download
	 * @param progress
	 *            the progress in percent
	 * @param speed
	 *            the speed in bytes/seconds
	 */
	public void updateDownloadProgressAndSpeed(int id, int progress, long speed) {
		if (null != this.textViewDownloadSpeed) {
			String text = super.getString(R.string.downloadSpeed, this.bytesToHumanSize(speed));
			this.textViewDownloadSpeed.setText(text);
		}

		final int viewId = DBInfos.arrayDBInfos[id].progressBarId;
		final ProgressBar progressBar = (ProgressBar) super.findViewById(viewId);
		progressBar.setProgress(progress);
	}

	/**
	 * The downloader thread tell the ui that is job is done.
	 */
	public void downloadComplete() {
		Button button = (Button) super.findViewById(R.id.downloadButtonCancel);
		button.setVisibility(View.GONE);
		button = (Button) super.findViewById(R.id.downloadButtonContinue);
		button.setVisibility(View.VISIBLE);
		this.textViewDownloadSpeed.setText("");
		DownloaderActivity.currentScreen = CurrentScreen.DOWNLOAD_DONE;

		System.runFinalization();
		System.gc();
	}

	private String bytesToHumanSize(long value) {
		final String[] sizes = super.getResources().getStringArray(R.array.byteSizeHumanDisplay);
		float humanSize = (float) value;
		int count = 0;

		while (humanSize > 1024) {
			humanSize = humanSize / 1024;
			if (count < sizes.length)
				count++;
		}

		return String.format("%1$.1f%2$s", humanSize, sizes[count]);
	}

	public static boolean isDownloadDone() {
		return DownloaderActivity.currentScreen == CurrentScreen.DOWNLOAD_DONE;
	}

	public void onClick(View v) {
		switch (v.getId()) {

		case R.id.downloadFirstRunButtonContinue:
			this.firstRunContinue(v);
			break;
		case R.id.dwnlNowButtonOk:
			this.dwnlNowClickButton(v);
			break;
		case R.id.downloadButtonContinue:
			this.showMainMenu(v);
			break;
		case R.id.dwnlNowCheckBoxDictDB:
		case R.id.dwnlNowCheckBoxKanjiDB:
			this.clickCheckBoxDownload(v);
			break;
		case R.id.dwnlNowButtonCancel:
		case R.id.downloadButtonCancel:
		case R.id.initErrorButtonQuit:
		case R.id.downloadFirstRunButtonQuit:
			this.quitApplication(v);
			break;
		}

	}
}
