/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.cupcake.kanji;


import org.vono.narau.cupcake.R;

import android.app.Activity;
import android.os.Bundle;
import android.text.Layout;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnLongClickListener;
import android.view.View.OnTouchListener;
import android.widget.ScrollView;
import android.widget.TextView;

/**
 * Display a list of kanjis pass as an extra with the TAG key, and return the
 * position of the character which had a long click on it.
 * 
 */
public class KanjiListLinksActivity extends Activity implements OnTouchListener, OnLongClickListener {
	public static final String TAG = KanjiListLinksActivity.class.getSimpleName();

	private TextView textView;
	private int position;
	private int[] codepoint;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Bundle extra = this.getIntent().getExtras();

		if (null != extra) {

			this.position = -1;

			this.textView = new TextView(this);
			this.textView.setOnTouchListener(this);
			this.textView.setOnLongClickListener(this);

			this.codepoint = extra.getIntArray(TAG);
			this.textView.setText(codepointToString());
			this.textView.setTextSize(25);

			ScrollView scrollView = new ScrollView(this);
			scrollView.addView(this.textView);
			super.setContentView(scrollView);
			super.setTitle(R.string.kanjiList);
		} else {
			Log.e(TAG, "no text to display");
		}
	}

	private String codepointToString() {
		final int len = this.codepoint.length;
		final int[] cp = this.codepoint;
		StringBuilder sb = new StringBuilder(len * 2);
		for (int i = 0; i < len; i++) {
			sb.append(Character.toChars(cp[i]));
			sb.append(" ");
		}

		return sb.toString();
	}

	public boolean onTouch(View view, MotionEvent mEvent) {
		// http://stackoverflow.com/questions/8637603/
		Layout layout = ((TextView) view).getLayout();
		int x = (int) mEvent.getX();
		int y = (int) mEvent.getY();

		if (layout != null) {
			int line = layout.getLineForVertical(y);
			this.position = layout.getOffsetForHorizontal(line, x);
		}

		return false;
	}

	public boolean onLongClick(View v) {
		super.setResult(this.position);
		super.finish();
		return true;
	}
}
